<?php
namespace Ak\Image\Processor\Adapter;

use Ak\Image\ImageInterface;
use Ak\Image\Processor\ResizeMode;

/**
 * Omogućava nizanje više adaptera.
 *
 * @package Ak\Image\Processor\Adapter
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/image-processor
 */
final class AdapterChain implements AdapterInterface
{
    /** @var AdapterInterface[] */
    private $adapters = [];

    /**
     * Dodaje dati adapter u lanac.
     *
     * @param AdapterInterface $adapter
     */
    public function addAdapter(AdapterInterface $adapter)
    {
        $this->adapters[] = $adapter;
    }

    /**
     * Vraća adaptere dodane u ovaj lanac.
     *
     * @return AdapterInterface[]
     */
    public function getAdapters(): array
    {
        return $this->adapters;
    }

    /**
     * Da li ovaj procesor može obraditi datu sliku?
     *
     * @param ImageInterface $image
     * @return bool
     */
    public function canProcessImage(ImageInterface $image): bool
    {
        foreach ($this->adapters as $adapter) {
            if ($adapter->canProcessImage($image)) {
                return true;
            }
        }

        return false;
    }

    /**
     * Optimizira datu sliku.
     *
     * @param ImageInterface $image
     * @return ImageInterface
     */
    public function optimize(ImageInterface $image): ImageInterface
    {
        foreach ($this->adapters as $adapter) {
            if ($adapter->canProcessImage($image)) {
                return $adapter->optimize($image);
            }
        }

        return clone $image;
    }

    /**
     * Mijenja veličinu slike.
     *
     * @param ImageInterface $image
     * @param int|null $width
     * @param int|null $height
     * @param ResizeMode|null $mode
     * @return ImageInterface
     */
    public function resize(ImageInterface $image, int $width = null, int $height = null, ResizeMode $mode = null): ImageInterface
    {
        foreach ($this->adapters as $adapter) {
            if ($adapter->canProcessImage($image)) {
                return $adapter->resize($image, $width, $height, $mode);
            }
        }

        return clone $image;
    }
}