<?php
namespace Ak\Image\Processor\Adapter;

use Ak\Image\Image;
use Ak\Image\ImageInterface;
use Ak\Image\ImagickAwareInterface;
use Ak\Image\Processor\ResizeMode;
use Imagick;

/**
 * Obrađuje slike koristeći Imagick library.
 *
 * @package Ak\Image\Processor\Adapter
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/image-processor
 */
final class ImagickAdapter implements AdapterInterface
{
    /** @var array */
    private $config;

    /**
     * Konstruktor.
     *
     * @param array $config
     */
    public function __construct(array $config)
    {
        $this->config = $config;
    }

    /**
     * Da li ovaj procesor može obraditi datu sliku?
     *
     * @param ImageInterface $image
     * @return bool
     */
    public function canProcessImage(ImageInterface $image): bool
    {
        return (bool) preg_match('#^image/(gif|jpeg|png)$#', $image->getMimeType());
    }

    /**
     * Optimizira datu sliku.
     *
     * @param ImageInterface $image
     * @return ImageInterface
     * @throws \ImagickException
     */
    public function optimize(ImageInterface $image): ImageInterface
    {
        if ($image instanceof ImagickAwareInterface) {
            $imagick = clone $image->getImagick();
        } else {
            $imagick = new Imagick();
            $imagick->readImageBlob($image->getContents());
        }

        $imagick->stripImage();

        $width = $image->getWidth();
        $height = $image->getHeight();

        $imagick->setImageCompressionQuality($this->config['compression_quality'] ?? 85);
        $imagick->thumbnailImage($width, $height);

        switch ($image->getMimeType()) {
            case 'image/jpeg':
                $imagick->setImageFormat('jpeg');
                $imagick->setSamplingFactors(array('2x2', '1x1', '1x1'));
                $profiles = $imagick->getImageProfiles("icc", true);
                $imagick->stripImage();

                if(!empty($profiles)) {
                    $imagick->profileImage('icc', $profiles['icc']);
                }

                $imagick->setInterlaceScheme(Imagick::INTERLACE_JPEG);
                $imagick->setColorspace(Imagick::COLORSPACE_SRGB);
                break;
            case 'image/png':
                $imagick->setImageFormat('png');
                break;
            case 'image/gif':
                $imagick->setImageFormat('gif');
                break;
        }

        $optimized = new Image($imagick);

        return $optimized;
    }

    /**
     * Mijenja veličinu slike.
     *
     * @param ImageInterface $image
     * @param int|null $width
     * @param int|null $height
     * @param ResizeMode|null $mode
     * @return ImageInterface
     * @throws \ImagickException
     */
    public function resize(ImageInterface $image, int $width = null, int $height = null, ResizeMode $mode = null): ImageInterface
    {
        if ($image instanceof ImagickAwareInterface) {
            $imagick = clone $image->getImagick();
        } else {
            $imagick = new Imagick();
            $imagick->readImageBlob($image->getContents());
        }

        $imageWidth = $imagick->getImageWidth();
        $imageHeight = $imagick->getImageHeight();
        $imageRatio = $imageWidth / $imageHeight;
        $bestfit = false;

        switch ($mode->getValue()) {
            case ResizeMode::SCALE:
                if ($width && $height) {
                    throw new \InvalidArgumentException("'scale' način smije zadati samo jednu dimenziju.");
                }

                if ($width) {
                    $newWidth = $width;
                    $newHeight = $width / $imageRatio;
                } else {
                    $newWidth = $height * $imageRatio;
                    $newHeight = $height;
                }
                break;
            case ResizeMode::FIT:
                $bestfit = true;
                $newWidth = $width;
                $newHeight = $height;
                break;
            case ResizeMode::THUMB:
            default:
                $newWidth = $width;
                $newHeight = $height;
                $newRatio = $newWidth / $newHeight;

                if ($imageRatio > 1) {
                    $cropWidth = $newRatio * $imageHeight;
                    $cropHeight = $imageHeight;
                } else if ($imageRatio < 1) {
                    $cropWidth = $imageWidth;
                    $cropHeight = $imageWidth / $newRatio;
                }

                if (isset($cropWidth) && isset($cropHeight)) {
                    $x = ($imageWidth - $cropWidth) / 2;
                    $y = ($imageHeight - $cropHeight) / 2;

                    $imagick->cropImage($cropWidth, $cropHeight, $x, $y);
                }
                break;
        }

        $imagick->resizeImage($newWidth, $newHeight, Imagick::FILTER_UNDEFINED, 1, $bestfit);
        $resized = new Image($imagick);

        return $this->optimize($resized);
    }
}